using System;
using System.Data;
using System.Data.SqlClient;
using System.Web.UI.WebControls;
using System.Collections.Generic;
using System.Drawing;

using SolarWinds.Logging;
using SolarWinds.Orion.Core.Common.Models;
using SolarWinds.Orion.Web.Charting;
using SolarWinds.Orion.Web.DAL;
using SolarWinds.Orion.Web.Controls;
using SolarWinds.Orion.Core.Common;
using SolarWinds.Orion.Common;
using SolarWinds.Orion.Web.UI;
using SettingsDAL = SolarWinds.Orion.Web.DAL.SettingsDAL;
using Limitation = SolarWinds.Orion.Web.Limitation;
using SolarWinds.Orion.Web;
using System.Web;

public partial class Orion_NetPerfMon_GroupMemberAggregateStatus : System.Web.UI.Page
{

	#region Data

	DataTable AggregateStatusTable, GroupDetailsTable, pie_parts;
	string groupName="";
	string groupID="";

	public string getGroupName() {
		return groupName;
	}

	public string getGroupID() {
		return groupID;
	}

	private void showError(Exception e) {
		AggregateStatusPanel.Visible=false;
		ErrorPanel.Visible=true;
		ErrorLabel.Text=e.Message;
	}

	private void showError(string errorMessage) {
		AggregateStatusPanel.Visible=false;
		ErrorPanel.Visible=true;
		ErrorLabel.Text=errorMessage;
	}

	// sql access
	protected override void OnLoad(EventArgs e)
	{
		string whereGroup = "";
		
		if (!String.IsNullOrEmpty(Request.QueryString["ContainerID"])) 
		{
			groupID = Request.QueryString["ContainerID"];
			whereGroup = " WHERE C.ContainerID='" + groupID + "' ";
		} else {
			showError("GroupID or ContainerID was not specified");
		}
		
		// Get info about the container that represents the groups
		string groupDetailsQuery = String.Format("SELECT concat('/Orion/NetPerfMon/ContainerDetails.aspx?NetObject=C:',C.ContainerID) as detailsURL, C.Name, C.ContainerID, C.Status FROM Containers C" + whereGroup);

		try {
			using (SqlCommand sqlCommand = SqlHelper.GetTextCommand(groupDetailsQuery))
			{
				GroupDetailsTable = SqlHelper.ExecuteDataTable(sqlCommand);
			}
		} catch (Exception ex) {
			showError("Could not query details for group with ContainerID " + groupID + ": " + ex.Message);
		}

		
		if (GroupDetailsTable.Rows.Count > 0) {
				DataRow[] groupSummaryRow = GroupDetailsTable.Select("ContainerID is not null","");
				groupName = groupSummaryRow[0]["Name"].ToString();
				Page.Title=groupName;
		} else {
			showError("No info available for ContainerID " + groupID);
		}


		// Get aggregate info about status of group members
		string aggregateStatusQuery = String.Format("SELECT GroupStatusSummary.Name as GroupName, SI.ShortDescription as Status, SI.StatusID as StatusID, GroupStatusSummary.detailsURL as detailsURL, '/Orion/images/StatusIcons/Small-' + SI.IconPostFix + '.gif' AS Icon, CASE WHEN GroupStatusSummary.CountStatus IS NULL THEN 0 WHEN GroupStatusSummary.CountStatus IS NOT NULL THEN CAST(CAST(GroupStatusSummary.CountStatus AS DECIMAL(8,2))/CAST(GroupStatusSummary.TotalMembers AS DECIMAL(8,2))*100 AS DECIMAL(8,2)) END as PercentStatus, GroupStatusSummary.detailsURL as [_Linkfor_PercentStatus] FROM StatusInfo SI LEFT OUTER JOIN (SELECT concat('/Orion/NetPerfMon/ContainerDetails.aspx?NetObject=C:',C.ContainerID) as detailsURL, CMS.Status, Count(CMS.Status) AS CountStatus,(SELECT count('x') from ContainerMemberSnapshots where ContainerID=C.ContainerID AND Status IN (0,1,2,3,14,17)) as TotalMembers, C.Name FROM Containers C INNER JOIN ContainerMemberSnapshots CMS on C.ContainerID = CMS.ContainerID " + whereGroup + " GROUP BY C.ContainerID, C.Name, CMS.Status) GroupStatusSummary on SI.StatusID = GroupStatusSummary.Status WHERE SI.StatusID IN (0,1,2,3,14,17) ORDER BY SI.StatusID DESC");

		try {
			using (SqlCommand sqlCommand = SqlHelper.GetTextCommand(aggregateStatusQuery ))
			{
				AggregateStatusTable = SqlHelper.ExecuteDataTable(sqlCommand);
			}

			// Add a row to AggregateStatusTable to hold totals
			DataRow summaryAggregateStatusTable = AggregateStatusTable.NewRow();
			summaryAggregateStatusTable ["PercentStatus"] = Convert.ToDecimal(AggregateStatusTable.Compute("Sum(PercentStatus)",""));
			summaryAggregateStatusTable ["GroupName"] = groupName;
			summaryAggregateStatusTable ["Status"] = "Total";
			AggregateStatusTable.Rows.Add(summaryAggregateStatusTable );

			// Add Color column for chart support
			AggregateStatusTable.Columns.Add("Color");

			// Add RowIndex to give us something to sort on 
			AggregateStatusTable.Columns.Add("RowIndex");
			int rowIndex = AggregateStatusTable.Rows.Count;

			// Populate Color, detailsURL, and RowIndex columns
			// detailsURL may be null so we must populate it
			foreach (DataRow r in AggregateStatusTable.Rows ) {

				rowIndex -= 1;
				r["RowIndex"]=rowIndex;

				r["detailsURL"]=GroupDetailsTable.Rows[0]["detailsURL"];

				switch (r["StatusID"].ToString()) {
					case "17":
						r["Color"]="#dddddd"; //"Could Not Poll"
						break;
					case "14":
						r["Color"]="#660000"; //"Critical" check_mk value was #f00
						break;
					case "3":
						r["Color"]="#ffff00"; //"Warning"check_mk value was #ff0
						break;
					case "2":
						r["Color"]="#ff0000"; //"Down"check_mk value was #048
						break;
					case "1":
						r["Color"]="#009900"; //"Up" check_mk value was #0b3
						break;
					case "0":
						r["Color"]="#cccccc"; //"Unknown"check_mk value was #f80
						break;
					default:
						r["Color"]=""; 
						break;
				}
			}
		} catch (Exception ex) {
			showError("Could not query aggregate stats for members of group with ContainerID " + groupID + ": " + ex.Message);
		}

		try {
			CalculatePieChart();
		} catch (Exception ex) {
			showError("Could not create pie chart for group " + groupName + ": " + ex.Message);
		}

		PublishData();
	} 

	// bind data to repeater
	private void PublishData()
	{
		AggregateStatusGrid.DataSource = AggregateStatusTable;
		AggregateStatusGrid.DataBind();

		pieChartJSGrid.DataSource = pie_parts;
		pieChartJSGrid.DataBind();

	}

	#endregion


	#region GroupMemberAggregateStatus

	protected override void OnLoadComplete(EventArgs e)
	{
		base.OnLoadComplete(e);
	}

	#endregion

	// Calculations for charts
	#region PieChart

	private void CalculatePieChart()
	{
		pie_parts = new DataTable("pie_parts");
		pie_parts.Columns.Add("Name");
		pie_parts.Columns.Add("Value");
		pie_parts.Columns.Add("StartPercent");
		pie_parts.Columns.Add("EndPercent");
		pie_parts.Columns.Add("Color");

		// Get the total number of statuses
		DataRow[] summaryRow = AggregateStatusTable.Select("Status='Total'","");
		int total = Convert.ToInt32(summaryRow[0]["PercentStatus"]);

		// Flip the status table so we can work over it in ascending status order
		DataRow[] sortedAggregateStatusTable = AggregateStatusTable.Select("StatusID>=0","StatusID");
		DataTable reverseAggregateStatusTable = sortedAggregateStatusTable.CopyToDataTable() ;

		if (total > 0)
		{
			// Count number of status categories with more than zero members
			int num_nonzero = 0;

			foreach (DataRow r in AggregateStatusTable.Rows ) {
				if (Convert.ToInt32(r["PercentStatus"]) > 0) {
					num_nonzero += 1;
				}
			}		

			double start_percent = 0.0;
			
			foreach (DataRow r in reverseAggregateStatusTable.Rows ) {
				// PercentStatus is an integer, but value must be a double for math to work
				double value = Convert.ToDouble(r["PercentStatus"]);
				string status = r["Status"].ToString();
				string color = r["Color"].ToString();

				DataRow pieRow = pie_parts.NewRow();

				pieRow["Name"]=status;
				pieRow["Value"]=value;
				pieRow["StartPercent"]=start_percent;
				pieRow["EndPercent"]=start_percent+value;
				pieRow["Color"]=color;
				pie_parts.Rows.Add(pieRow);
				
				// compute where to start the position of the next "slice"
				start_percent += value;
			}
		}
	}

	#endregion
}